/****************************************************************************
** $Id: quickdispatchobject.cpp  20030513   edited May 9 13:12 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "quickdispatchobject.h"
#include "quickinterpreter.h"
#include "qsenv.h"
#include <private/qpluginmanager_p.h>
#include <qobject.h>
#include <qmetaobject.h>
#include <qwidget.h>

QuickPtrDispatchObject::QuickPtrDispatchObject( const char *n, void *p )
    : QuickUnnamedObject( 0, n ), ptr( p )
{
}

QuickPtrDispatchObject::~QuickPtrDispatchObject()
{
}

void QuickPtrDispatchObject::setPointer( void *p )
{
    ptr = p;
}

bool QuickDispatchObjectFactory::constructInterface( const QCString &className,
						     void *ptr,
						     QPtrVector<QObject> &result )
{
    Q_ASSERT( resultVector == 0 );
    resultVector = &result;
    bool ret = createInterface( className, ptr );
    resultVector = 0;
    return ret;
}

bool QuickDispatchObjectFactory::constructInstance( const QString &className,
					    const QValueList<QVariant> &args,
					    QPtrVector<QObject> &result )
{
    Q_ASSERT( resultVector == 0 );
    int oldIdx = result.size();
    resultVector = &result;
    bool ret = createInstance( className, args );
    resultVector = 0;
    if ( !ret ) {
	for ( uint i = oldIdx; i < result.size(); ++i )
	    result.insert( i, 0 );
    }
    return ret;
}

void QuickDispatchObjectFactory::throwError( const QString &message ) const
{
    QString msg( message );
    ip->env()->throwError( message );
}

void QuickDispatchObjectFactory::addInterface( QObject *iface )
{
    Q_ASSERT( iface );
    Q_ASSERT( !iface->isWidgetType() );
    Q_ASSERT( !iface->parent() );
    Q_ASSERT( resultVector );
    interpreter()->insertChild( iface );
    int idx = resultVector->size();
    resultVector->resize( idx + 1 );
    resultVector->insert( idx, iface );
}

void QuickDispatchObjectFactory::addObject( QObject *obj )
{
    Q_ASSERT( obj );
    Q_ASSERT( resultVector );
    Q_ASSERT( resultVector->isEmpty() );
    int idx = resultVector->size();
    resultVector->resize( idx + 1 );
    resultVector->insert( idx, obj );
}

QuickInterpreter* QuickUnnamedObject::interpreter() const
{
    Q_ASSERT( parent() && parent()->inherits( "QuickInterpreter" ) );
    return (QuickInterpreter*)parent();
}

void QuickUnnamedObject::throwError( const QString &message ) const
{
    interpreter()->env()->throwError( message );
}
