/****************************************************************************
** $Id: qsinterfacefactory.cpp  20030513   edited May 9 15:25 $
**
** Copyright (C) 2001-2002 Trolltech AS.  All rights reserved.
**
** This file is part of the Qt Script for Applications framework (QSA).
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding a valid QSA Beta Evaluation Version license may use
** this file in accordance with the QSA Beta Evaluation Version License
** Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
**   information about QSA Commercial License Agreements.
** See http://www.trolltech.com/gpl/ for GPL licensing information.
**
** Contact info@trolltech.com if any conditions of this licensing are
** not clear to you.
**
*****************************************************************************/

#include "qsinterfacefactory_p.h"
#include <quickinterpreter.h>
#include <qsobject.h>
#include <qsenv.h>
#include <quickobjects.h>
#include "qsinterpreter.h"


QSInterfaceFactory::QSInterfaceFactory( QSInterpreter *i )
    : QuickDispatchObjectFactory( i->interpreter() ), interp( i ), dirty( FALSE )
{
    i->interpreter()->addDispatchObjectFactory( this );
}


bool QSInterfaceFactory::isConstructor( const QString &className ) const
{
    if( dirty )
	( (QSInterfaceFactory*) this )->createClassList();
    return instDescr.contains( className );
}


QMap<QString,QString> QSInterfaceFactory::instanceDescriptors() const
{
    if( dirty )
	( (QSInterfaceFactory*) this )->createClassList();
    return instDescr;
}


QMap<QString,QObject*> QSInterfaceFactory::staticDescriptors() const
{
    if( dirty )
	( (QSInterfaceFactory*) this )->createClassList();
    return statDescr;
}


bool QSInterfaceFactory::createInterface( const QCString &className, void *ptr )
{
    QMap<QString, QSWrapperFactory* >::ConstIterator it = wrapperFactoryCache.find( className );
    QSWrapperFactory *f = 0;
    if ( it != wrapperFactoryCache.end() ) {
	f = *it;
    } else {
	for ( QSWrapperFactory *w = wrappers.first(); w; w = wrappers.next() ) {
	    QStringList wrps = w->classes();
	    if ( wrps.find( className ) != wrps.end() ) {
		f = w;
		wrapperFactoryCache.replace( className, f );
		break;
	    }
	}
    }

    if ( f ) {
	QObject *o = f->create( className, ptr );
	if ( o ) {
	    addInterface( o );
	    return TRUE;
	}
    }

    return FALSE;
}


bool QSInterfaceFactory::createInstance( const QString &className,
				    const QValueList<QVariant> &args )
{
    QSArgumentList qsArgs;
    for ( QValueList<QVariant>::ConstIterator it = args.begin(); it != args.end(); ++it ) {
	if ( (*it).type() == QVariant::String ) {
		static int length_of_Pointer = 7;
		static QString pointer_header = "Pointer";
		QString s = (*it).toString();
		if ( s.left( length_of_Pointer ) == pointer_header ) {
		    QStringList l = QStringList::split( ':', s );
		    if ( l.count() == 3 ) {
			if ( l[2] != "QObject" ) {
			    ulong lng = l[1].toULong();
			    void *ptr = (void*)lng;
			    qsArgs.append( QSArgument( ptr ) );
			} else {
			    ulong lng = l[1].toULong();
			    QObject *o = (QObject*)lng;
			    qsArgs.append( QSArgument( o ) );
			}
			continue;
		    }
		}
	}
	qsArgs.append( QSArgument( *it ) );
    }

    QMap<QString, QSObjectFactory* >::ConstIterator mit = objectFactoryCache.find( className );
    QSObjectFactory *f = 0;
    if ( mit != objectFactoryCache.end() ) {
	f = *mit;
    } else {
	for ( QSObjectFactory *c = ( (QSInterfaceFactory*)this )->ctors.first(); c;
	      c = ( (QSInterfaceFactory*)this )->ctors.next() ) {
	    QStringList cls = c->instanceDescriptors().keys();
	    if ( cls.find( className ) != cls.end() ) {
		f = c;
		( (QSInterfaceFactory*)this )->objectFactoryCache.replace( className, f );
		break;
	    }
	}
    }

    if ( f ) {
	QObject *ctx = 0;
	QSObject obj = interpreter()->env()->currentScope();
	if ( obj.isA( interpreter()->wrapperClass() ) )
	    ctx = interpreter()->wrapperClass()->shared( &obj )->objects[0];
	QObject *o = f->create( className, qsArgs, ctx );
	if ( o ) {
	    addObject( o );
	    return TRUE;
	}
    }

    return FALSE;
}


void QSInterfaceFactory::createClassList()
{
    if ( !dirty )
	return;
    instDescr.clear();
    statDescr.clear();
    for ( QSObjectFactory *c = ctors.first(); c; c = ctors.next() ) {
	QMap<QString,QString> idesc = c->instanceDescriptors();
	for( QMap<QString,QString>::ConstIterator insts = idesc.begin();
	     insts != idesc.end(); ++insts )
	    instDescr[ insts.key() ] = *insts;
	QMap<QString,QObject*> sdesc = c->staticDescriptors();
	for( QMap<QString,QObject*>::ConstIterator stats = sdesc.begin();
	     stats != sdesc.end(); ++stats )
	    statDescr[ stats.key() ] = *stats;
    }
//     for ( QSWrapperFactory *w = wrappers.first(); w; w = wrappers.next() )
// 	QStringList names = w->classes();
}

void QSInterfaceFactory::addConstructor( QSObjectFactory *ctor )
{
    ctors.append( ctor );
    dirty = TRUE;
    objectFactoryCache.clear();
    if( interp->project() )
	interp->project()->invalidateProject();
}

void QSInterfaceFactory::addWrapper( QSWrapperFactory *w )
{
    wrappers.append( w );
    dirty = TRUE;
    wrapperFactoryCache.clear();
    if( interp->project() )
	interp->project()->invalidateProject();
}

#include <qmetaobject.h>
